import matplotlib
import matplotlib.pyplot as plt
import numpy as np
from metabolism.Helper import Helper
import seaborn as sns
import colorsys
from matplotlib.colors import ListedColormap
import matplotlib.colors as mcolors
from matplotlib import cm
import SimpleITK as sitk

class Painter:
    def __init__(self):
        self.helper = Helper()
        pass
    def hotmap_zscore(self,matrix):
        plt.figure(figsize=(4,4))
        mask = np.triu(np.ones_like(matrix,dtype=bool))
        sns.heatmap(
            matrix,
            annot=False,          # 在格子中显示数值
            fmt=".2f",           # 数值格式（保留两位小数）
            mask=mask,
            cmap='coolwarm',     # 颜色映射
            vmin=-3, vmax=3,     # 颜色范围
            square=True,         # 保持格子为正方形
            linewidths=0.5,      # 格子间线宽
            cbar=True,
            cbar_kws={'label': 'Correlation'}
        )
        plt.tight_layout()
        plt.show()
    def hotmap_zscore2(self,matrix1,matrix2,matrixtype):
        '''
            - matrixtype: "z-score" or "kldivergence"
        '''
        mask_lower = np.triu(np.ones_like(matrix1, dtype=bool), k=1)  # 隐藏上三角（含对角线）
        mask_upper = np.tril(np.ones_like(matrix2, dtype=bool), k=-1)  # 隐藏下三角（含对角线）
        fig, ax = plt.subplots(figsize=(5,4))
        if matrixtype == "z-score":
            vmin,vmax = -3,3
        elif matrixtype == "kldivergence":
            vmin,vmax = 0,1
        else:
            vmin,vmax = None,None
            print(f"matrixtype is wrong ! z-score or kldivergence !! ")
            # return None
        sns.heatmap(
            matrix1,
            mask=mask_lower,          # 隐藏上三角和对角线
            cmap='coolwarm',             # 下三角用蓝色系
            vmin=vmin, vmax=vmax,     # 颜色范围
            annot=False,
            fmt=".2f",
            square=True,
            linewidths=0.5,
            cbar=True,
            ax=ax
        )
        
        sns.heatmap(
            matrix2,
            mask=mask_upper,          # 隐藏下三角和对角线
            cmap='coolwarm',              # 上三角用红色系
            vmin=vmin, vmax=vmax,     # 颜色范围
            annot=False,
            fmt=".2f",
            square=True,
            linewidths=0.5,
            cbar=False,
            ax=ax
        )
        ax.plot(
            [matrix1.shape[0] + 1, 0],
            [matrix1.shape[0] + 1, 0],          # x从i到i+1
            color='red',         # 红色
            linewidth=2,         # 线宽
            transform=ax.transData,  # 使用数据坐标系
            clip_on=False,         # 避免边缘裁剪
            alpha=0.5
        )
        
        plt.tight_layout()
        plt.show()

    def show_slices(self,suv_volume, roi_volume, x=None, y=None, z=None):
        suv_volume = self.helper.resample_to_isotropic(suv_volume, new_spacing=[2.0, 2.0, 2.0],interpolator=sitk.sitkLinear)
        roi_volume = self.helper.resample_to_isotropic(roi_volume, new_spacing=[2.0, 2.0, 2.0],interpolator=sitk.sitkNearestNeighbor)
        suv_volume = sitk.GetArrayFromImage(suv_volume)
        roi_volume = sitk.GetArrayFromImage(roi_volume)
        
        roi_labels = np.unique(roi_volume)
        roi_labels = roi_labels[roi_labels != 0]
        
        colors = cm.get_cmap('tab20', len(roi_labels))
        fig, axes = plt.subplots(1, 3, figsize=(15, 5))
        
        slices = [
            ("Axial",   roi_volume[x, :, :] if x is not None else roi_volume.shape[0]//2, x),
            ("Coronal", roi_volume[:, y, :] if y is not None else roi_volume.shape[1]//2, y),
            ("Sagittal",roi_volume[:, :, z] if z is not None else roi_volume.shape[2]//2, z)
        ]
        
        for ax, (title, slc, pos) in zip(axes, slices):
            suv_slice = suv_volume[pos, :, :] if title == "Axial" else \
                        suv_volume[:, pos, :] if title == "Coronal" else \
                        suv_volume[:, :, pos]
            ax.imshow(suv_slice.T, cmap="gray", origin="lower")
            
            for i, label in enumerate(roi_labels):
                mask = (slc == label).T
                color = colors(i)
                ax.contour(mask, colors=[color], linewidths=1.2, levels=[0.5])
                
            ax.set_title(f"{title} View\nPosition: {pos}")
            ax.axis("off")
    
        legend_elements = [plt.Line2D([0], [0], color=colors(i), lw=2, label=f'ROI {label}') 
                          for i, label in enumerate(roi_labels)]
        
        plt.tight_layout()
        plt.show()
    def hotmap_edge(self,matrix):
        plt.figure(figsize=(4,4))
        mask = np.triu(np.ones_like(matrix,dtype=bool))
        vmax = np.max(matrix)  # 获取数据最大值
        sns.heatmap(
            matrix,
            annot=False,          # 在格子中显示数值
            fmt=".2f",           # 数值格式（保留两位小数）
            mask=mask,
            cmap='coolwarm',     # 颜色映射
            square=True,         # 保持格子为正方形
            linewidths=0.5,      # 格子间线宽
            cbar=True,
            cbar_kws={'label': 'Correlation'}
        )
        plt.tight_layout()
        plt.show()


import numpy as np
import pandas as pd
import scipy.stats as stats
from scipy.stats import mannwhitneyu
from scipy.stats import ks_2samp
from scipy.stats import brunnermunzel
from scipy.stats import permutation_test
from scipy.stats import ttest_ind

from scipy.sparse import csr_matrix
from scipy.sparse.linalg import eigsh
from sklearn.cluster import KMeans
from scipy.sparse.csgraph import laplacian
import networkx as nx
from networkx.algorithms.community import modularity
from metabolism.CorrelationAnalyzer import CorrelationAnalyzer
from metabolism.Helper import Helper

class SingleAnalyzer:
    def __init__(self,control,patient,modelname,suvrelative="brain"):
        self.ERROR_ICON = '❌'
        self.SUCCESS_ICON = '✅'
        self.RUNNING_ICON = '⏳'
        self.INFO_ICON = '➤'
        self.control = control
        self.patient = patient
        self.modelname = modelname
        self.suvrelative = suvrelative
        self.correlationAnalyzer = CorrelationAnalyzer()
        self.helper = Helper()
    def process_group2one(self):
        control = self.control
        patient = self.patient
        modelname = self.modelname
        df1 = self.correlationAnalyzer.dataset2df(control,modelname)
        df2 = self.correlationAnalyzer.dataset2df(patient,modelname)
        self.control_suv = df1
        self.patient_suv = df2
        print(f"[{self.INFO_ICON}] control:",control,f"count:{len(df1)}")
        print(f"[{self.INFO_ICON}] patient:",patient,f"count:{len(df2)}")
        brainzone = df1.columns.values
        brainzone = [b for b in brainzone if b not in ["Age","Sex","Weight"]]
        if self.suvrelative == "brain":
            row_means = df1[brainzone].mean(axis=1)
        if self.suvrelative == "brainstem":
            row_means = df1[["Brainstem"]].mean(axis=1)
        df1[brainzone] = df1[brainzone].div(row_means, axis=0)
        if self.suvrelative == "brain":
            row_means = df2[brainzone].mean(axis=1)
        if self.suvrelative == "brainstem":
            row_means = df2[["Brainstem"]].mean(axis=1)
        df2[brainzone] = df2[brainzone].div(row_means, axis=0)
        self.control_suvr = df1
        self.patient_suvr = df2
        control_result = {}
        for key in df1.columns:
            if key in ["Age","Sex","Weight"]:
                continue
            if self.suvrelative == "brainstem":
                if key == 'Brainstem':
                    continue
            d1 = df1[key].values
            mu, sigma = np.mean(d1), np.std(d1)
            ks_stat, ks_p = stats.kstest(d1, 'norm', args=(mu, sigma))
            if ks_p < 0.05:
                print(f"{key}不服从正态分布")
            d1 = df1[key].values
            df_avg = np.mean(d1)
            df_min = np.mean(d1)
            control_result[key] = {"mean":np.mean(d1),"std":np.std(d1),"min":np.min(d1),"max":np.max(d1)}
        patient_result = []
        for idx in range(len(df2)):
            tmp = df2.iloc[idx]
            oneresult = {}
            for key in control_result:
                d2 = tmp[key]
                oneresult[key] = (d2 - control_result[key]["mean"]) / control_result[key]["std"]
            patient_result.append(oneresult)
        return control_result, patient_result

class PairAnalyzer:
    def __init__(self):
        self.ERROR_ICON = '❌'
        self.SUCCESS_ICON = '✅'
        self.RUNNING_ICON = '⏳'
        self.INFO_ICON = '➤'

    def intersubject_dissimilarity(self,matrixs,roi_names):
        print(f"[{self.INFO_ICON}] Dissimilarity -1 to 1.")
        matrixs = np.array(matrixs)
        dissimilarity_mean = np.zeros_like(roi_names)
        dissimilarity_std = np.zeros_like(roi_names)
        for i,roi_name in enumerate(roi_names):
            corr = np.corrcoef(matrixs[:,i])
            dissimilarity_mean[i] = np.mean(-corr)
            dissimilarity_std[i] = np.std(-corr)
        df = pd.DataFrame({"roi":roi_names,"dissimilarity_mean":dissimilarity_mean,"dissimilarity_std":dissimilarity_std})
        print(f"[{self.SUCCESS_ICON}] df columns: roi, dissimilarity_mean, dissimilarity_std")
        return df

    def roipair_compare_group4group(self,matrixs1,matrixs2,roi_names,pmethod="mannwhitneyu"):
        print(f"[{self.INFO_ICON}] If roi-roi pair has significant difference between two groups.")
        if len(matrixs2) <=1:
            print(f"[{self.ERROR_ICON}] matrixs2 has one data, Please use function 'roipair_compare_group4one' !! ")
            return None
        matrixs1 = np.array(matrixs1)
        matrixs2 = np.array(matrixs2)
        results = []
        for i in range(len(roi_names)):
            for j in range(i+1,len(roi_names)):
                data1 = matrixs1[:,i,j]
                data2 = matrixs2[:,i,j]
                if pmethod == "mannwhitneyu":
                    stat, p = mannwhitneyu(data1, data2, alternative='two-sided')
                elif pmethod == "kolmogoerov_smirnov":
                    stat, p = ks_2samp(data1, data2)
                elif pmethod == "brunnermunzel":
                    stat, p = brunnermunzel(data1, data2)
                elif pmethod == "t":
                    stat, p = ttest_ind(data1,data2, equal_var=False)
                row = {
                    'roi1': roi_names[i],
                    'roi2': roi_names[j],
                    'group1':np.mean(data1),
                    'group2':np.mean(data2),
                    'p': p,
                }
                results.append(row)
        df = pd.DataFrame(results)
        total_comparisons = len(df)  # ROI对的总数
        corrected_alpha = 0.05 / total_comparisons
        df['significant'] = df['p'] < corrected_alpha
        return df

    def roipair_compare_group4one(self,matrixs1,matrixs2,roi_names):
        print(f"[{self.INFO_ICON}] When matrix2 is a single data, you can calculate its distance from the mean of matrixs1 ")
        matrixs1 = np.array(matrixs1)
        matrixs2 = np.array(matrixs2)
        results = []
        for i in range(len(roi_names)):
            for j in range(i+1,len(roi_names)):
                data1 = matrixs1[:,i,j]
                data2 = matrixs2[0,i,j]
                row = {
                    'roi1': roi_names[i],
                    'roi2': roi_names[j],
                    'group1_mean':np.mean(data1),
                    'group1_std':np.std(data1),
                    'data2':data2,
                    'z-score':(data2-np.mean(data1)) / np.std(data1)
                }
                results.append(row)
        df = pd.DataFrame(results)
        df = df.sort_values(
            by="z-score",
            key=lambda x: x.abs(),  
            ascending=False        
        )
        return df

    def roi_compare_group4one(self,matrixs1,matrixs2,roi_names):
        print(f"[{self.INFO_ICON}] When matrix2 is a single data, you can calculate its distance from the mean of matrixs1 ")
        matrixs1 = np.array(matrixs1)
        matrixs2 = np.array(matrixs2)
        results = []
        for i in range(len(roi_names)):
            data1 = np.mean(matrixs1[:,i],axis=0)
            data2 = np.mean(matrixs2[0,i])
            row = {
                'roi': roi_names[i],
                'group1_mean':np.mean(data1),
                'group1_std':np.std(data1),
                'data2':data2,
                'z-score':(data2-np.mean(data1)) / np.std(data1)
            }
            results.append(row)
        df = pd.DataFrame(results)
        df = df.sort_values(
            by="z-score",
            key=lambda x: x.abs(),  
            ascending=False        
        )
        return df
        
class GraphAnalyzer:
    def __init__(self):
        self.ERROR_ICON = '❌'
        self.SUCCESS_ICON = '✅'
        self.RUNNING_ICON = '⏳'
        self.INFO_ICON = '➤'

    def feature_extract_from_matrix(self,matrix,roi_name):
        degree =self.node_degree(matrix)
        clustering_coeff = self.weighted_clustering_coeff(matrix)
        centrality = self.eigenvector_centrality(matrix)
        self.spectral_clustering(matrix)
        mask_lower = np.triu(np.ones_like(matrix, dtype=bool), k=1)
        feat = matrix[mask_lower].flatten()
        feat_name = np.array([f"{roi_name[i]}_{roi_name[j]}" for i in range(0,len(roi_name)) for j in range(i+1,len(roi_name))])
        feat = np.concatenate([feat,degree, clustering_coeff,centrality])
        feat_name = np.concatenate([feat_name,
                                    [f"{roi}_degree" for roi in roi_name],
                                    [f"{roi}_clustering" for roi in roi_name],
                                    [f"{roi}_centrality" for roi in roi_name]])
        return feat,feat_name
    def node_degree(self,matrix):
        print(f"[{self.RUNNING_ICON}] node degree (节点度) ...")
        degree = np.sum(matrix,axis=0)
        print(f"[{self.SUCCESS_ICON}] node degree (节点度) ...")
        return degree
    def spectral_clustering(self,matrix):
        print(f"[{self.RUNNING_ICON}] spectral clustering (谱聚类) ...")
        L = laplacian(matrix, normed=True) 
        eigenvaLlues, eigenvectors = np.linalg.eigh(L)
        k = 3  # 预设社区数量
        best_Q = -1
        best_k = -1
        for k in range(3,15):
            top_k_vectors = eigenvectors[:, :k]  
            kmeans = KMeans(n_clusters=k).fit(top_k_vectors)
            labels = kmeans.labels_ 
            G = nx.from_numpy_array(matrix)
            communities = []
            for cluster_id in np.unique(labels):
                nodes = np.where(labels == cluster_id)[0].tolist()
                communities.append(set(nodes)) 
            Q = modularity(G, communities)
            if best_Q < Q:
                best_Q = Q
                best_k = k
        if best_Q < 0.3:
            print(f"[{self.ERROR_ICON}] 不存在显著社区 ")
        else:
            print(f"[{self.SUCCESS_ICON}] 存在显著社区 Best K: {best_k} || Best Q: {round(best_Q,2)} ")
    
    def weighted_clustering_coeff(self,matrix):
        print(f"[{self.RUNNING_ICON}] weighted clustering coeff (加权聚类系数) ...")
        """
        处理连续权重无向图的聚类系数
        :param matrix: 对称矩阵，对角线为0
        :return: 各节点的加权聚类系数（0~1）
        """
        n = matrix.shape[0]
        np.fill_diagonal(matrix, 0)  # 确保无自环
        W = np.copy(matrix)
        
        # 计算二值化度数
        binary_adj = (W > 0).astype(int)
        degrees = np.sum(binary_adj, axis=1)
        
        coeffs = np.zeros(n)
        for i in range(n):
            if degrees[i] < 2:
                continue
                
            # 获取邻居索引
            neighbors = np.where(binary_adj[i] > 0)[0]
            
            # 计算所有邻居对的三元乘积
            triple_sum = 0.0
            for j in range(len(neighbors)):
                for k in range(j+1, len(neighbors)):
                    n1, n2 = neighbors[j], neighbors[k]
                    triple_prod = W[i,n1] * W[i,n2] * W[n1,n2]
                    triple_sum += np.cbrt(triple_prod)  # 立方根处理
                    
            coeffs[i] = 2 * triple_sum / (degrees[i] * (degrees[i]-1))
        print(f"[{self.SUCCESS_ICON}] weighted clustering coeff (加权聚类系数) ...")
        return coeffs

    def eigenvector_centrality(self,matrix):
        print(f"[{self.RUNNING_ICON}] centrality (特征向量中心性) ...")
        """
        处理连续权重矩阵的特征向量中心性
        :param matrix: 对称权重矩阵
        :return: 归一化的特征向量中心性
        """
        # 对称性检查
        if not np.allclose(matrix, matrix.T):
            matrix = (matrix + matrix.T) / 2
        vals, vecs = eigsh(matrix, k=1, which='LM')
        centrality = np.abs(vecs[:,0])
        print(f"[{self.SUCCESS_ICON}] centrality (特征向量中心性) ...")
        return (centrality - centrality.min()) / (centrality.max() - centrality.min())


import SimpleITK as sitk


def adjust_image_direction(image, desired_direction,verbose=True):
    """
    调整图像的方向到所需方向，如果需要的话会翻转图像。
    
    :param image: SimpleITK图像对象
    :param desired_direction: 一个包含九个元素的元组，表示所需的图像方向
    :return: 调整方向后的图像
    """
    # 获取当前图像的方向
    current_direction = image.GetDirection()
    if len(current_direction) == 16:
        input_array = np.array(current_direction)
        matrix_4x4 = input_array.reshape((4, 4))
        sub_matrix_3x3 = matrix_4x4[:3, :3]
        current_directiony = sub_matrix_3x3.flatten()

    # 计算需要翻转的轴
    flip_axes = [current_direction[i] * desired_direction[i] < 0 for i in [0,4,8]]
    if verbose:
        print("the axis need to be flip?:", flip_axes)
    # 翻转图像
    adjusted_image = sitk.Flip(image, flip_axes)

    # 设置新的方向
    adjusted_image.SetDirection(desired_direction)

    return adjusted_image
def resampleVolume(outspacing, vol,resamplemethod=sitk.sitkNearestNeighbor):
    """
    将体数据重采样的指定的spacing大小\n
    paras：
    outpacing：指定的spacing，例如[1,1,1]
    vol：sitk读取的image信息，这里是体数据\n
    return：重采样后的数据
    """
    outsize = [0, 0, 0]
    # 读取文件的size和spacing信息
    inputsize = vol.GetSize()
    inputspacing = vol.GetSpacing()
 
    transform = sitk.Transform()
    transform.SetIdentity()
    # 计算改变spacing后的size，用物理尺寸/体素的大小
    outsize[0] = round(inputsize[0] * inputspacing[0] / outspacing[0])
    outsize[1] = round(inputsize[1] * inputspacing[1] / outspacing[1])
    outsize[2] = round(inputsize[2] * inputspacing[2] / outspacing[2])
 
    # 设定重采样的一些参数
    resampler = sitk.ResampleImageFilter()
    resampler.SetTransform(transform)
    resampler.SetInterpolator(resamplemethod)
    resampler.SetOutputOrigin(vol.GetOrigin())
    resampler.SetOutputSpacing(outspacing)
    resampler.SetOutputDirection(vol.GetDirection())
    resampler.SetSize(outsize)
    newvol = resampler.Execute(vol)
    return newvol

def resample_image(input_image, reference_image,default=0):
    # 创建一个重采样过滤器
    resampler = sitk.ResampleImageFilter()
    resampler.SetReferenceImage(reference_image)
    
    # 设置插值方法，通常使用线性插值
    resampler.SetInterpolator(sitk.sitkNearestNeighbor)
    
    # 设置输出使用参考图像的spacing, origin和direction
    resampler.SetOutputSpacing(reference_image.GetSpacing())
    resampler.SetOutputOrigin(reference_image.GetOrigin())
    resampler.SetOutputDirection(reference_image.GetDirection())
    
    # 设置输出图像的size
    resampler.SetSize(reference_image.GetSize())
    resampler.SetDefaultPixelValue(default)
    
    # 执行重采样
    return resampler.Execute(input_image)


from metabolism.DcmWorker import DcmWorker
from metabolism.Segmentor import Segmentor
from metabolism.Helper import Helper
from metabolism.StabilityTester import StabilityTester
from metabolism.CorrelationAnalyzer import CorrelationAnalyzer
from metabolism.Analyzer import PairAnalyzer,GraphAnalyzer,SingleAnalyzer
from metabolism.MetabolicNetworks import GroupLevelNetwork,IndividualNetwork,KLNetwork
from metabolism.Painter import Painter
from metabolism.Classifier import Classifier






import torch,monai
import sys,os,json
import SimpleITK as sitk
import numpy as np
import torch.nn as nn
from metabolism.MPUM.datautils import resampleVolume,adjust_image_direction
from metabolism.MPUM.mpummodel import MPUM
from metabolism.MPUM.categories import prediction as label215

from tqdm import tqdm
from collections import OrderedDict
current_dir = os.path.dirname(os.path.abspath(__file__))
def resample_image(input_image, reference_image):
    def dataset_config_to_rulematrix(x):
        with open(x,"r") as f:
            x = json.load(f)
        rules = torch.zeros(len(x),215)
        for key in x:
            rules[x[key]["labelindex"]][x[key]["modelindex"]] = 1
        rules[0] = 1 - torch.sum(rules[1:],dim=0)
        return rules
    
    # 创建一个重采样过滤器
    resampler = sitk.ResampleImageFilter()
    resampler.SetReferenceImage(reference_image)
    
    # 设置插值方法，通常使用线性插值
    resampler.SetInterpolator(sitk.sitkLinear)
    
    # 设置输出使用参考图像的spacing, origin和direction
    resampler.SetOutputSpacing(reference_image.GetSpacing())
    resampler.SetOutputOrigin(reference_image.GetOrigin())
    resampler.SetOutputDirection(reference_image.GetDirection())
    
    # 设置输出图像的size
    resampler.SetSize(reference_image.GetSize())
    resampler.SetDefaultPixelValue(0)
    
    # 执行重采样
    return resampler.Execute(input_image)
import os
import SimpleITK as sitk

def safe_read_ct(ct_path):
    """安全读取NIFTI文件并处理方向矩阵问题"""
    if not os.path.exists(ct_path):
        raise FileNotFoundError(f"CT文件不存在: {ct_path}")

    reader = sitk.ImageFileReader()
    
    try:
        # 配置基础参数
        reader.SetFileName(ct_path)
        reader.LoadPrivateTagsOn()
        
        # 强制指定NIFTI处理器（需在ReadImageInformation之前）
        reader.SetImageIO("NiftiImageIO")  
        reader.ReadImageInformation()  # 预读元数据
        
        # 设置方向强制正交参数
        image_io = reader.GetImageIO()
        if image_io.GetName() == "NiftiImageIO":
            image_io.SetParameter("enforceOrientation", "true")
        
        # 执行实际读取
        return reader.Execute()
    
    except RuntimeError as e:
        # 处理已知的ITK方向矩阵异常
        if "orthonormal direction cosines" in str(e):
            print(f"警告: 方向矩阵非正交，尝试强制修正: {ct_path}")
            return _force_orthogonal(ct_path)
        raise
    
    finally:
        del reader  # 显式释放资源

def _force_orthogonal(ct_path):
    """备选修正方案：通过nibabel修正方向"""
    try:
        import nibabel as nib
        img = nib.load(ct_path)
        corrected_img = nib.as_closest_canonical(img)
        return sitk.GetImageFromArray(corrected_img.get_fdata())
    except ImportError:
        raise RuntimeError
def inference(config, nii_path,output_seg_path,output_stdct_path=None,check=True,modelname=None,dataset_mapping_to_model=None):
    '''
        dataset_mapping_to_model: .json file path
    '''
    config["tissuenumber"] = 215
    ct_path = nii_path
    # assert "." not in output_seg_path, "output_seg_path should be a dir path, not a file path"
    
    # print(f"提供的NIfTI路径是:{ct_path}")
    # file_path = os.path.dirname(os.path.abspath(__file__))
    try:
        orict_itk = sitk.ReadImage(os.path.join(ct_path))
        ct_itk = sitk.ReadImage(os.path.join(ct_path))
        # ct = safe_read_ct(ct_itk)
    except Exception as e:
        print(e)
        return None
    
    # print("----------------direction check and spacing check------------------------")

    # print("before processing, spacing:",ct_itk.GetSpacing())
    # print("before processing, direction:",ct_itk.GetDirection())
    
    new_direction = (-1, 0, 0, 0, -1, 0, 0, 0, 1)
    new_spacing = (2,2,2)  # 保持间距不变
    
    direction_check = np.mean(np.abs(np.array(ct_itk.GetDirection()) - np.array(new_direction)))
    spacing_check = np.mean(np.abs(np.array(ct_itk.GetSpacing()) - np.array([2,2,2])))

    # print("----------------pre-process <LUCIDA Standard Protocol>------------------------")
    
    ct_itk = resampleVolume(new_spacing,ct_itk,resamplemethod=sitk.sitkLinear)
    ct_itk = adjust_image_direction(ct_itk, new_direction)

    # print("after processing, spacing:",ct_itk.GetSpacing())
    # print("after processing, direction:",ct_itk.GetDirection())

    if output_stdct_path is not None:
        output_stdct_path_ = os.path.dirname(output_stdct_path)
        if not os.path.exists(output_stdct_path_):
            os.makedirs(output_stdct_path_)
            print(f"目录已创建：{output_stdct_path_}")
        sitk.WriteImage(ct_itk, output_stdct_path)
        print("standard protocol nii has been write in ",output_stdct_path)
    else:
        print("if need to save CT.nii.gz file in standard protocol (1.5mm), use arg <output_stdct_path>")
    
    def scale_intensity_range(ct, a_min, a_max, b_min, b_max, clip):
        if clip:
            ct = torch.clamp(ct, min=a_min, max=a_max)
        # 线性缩放
        ct = (ct - a_min) / (a_max - a_min) * (b_max - b_min) + b_min
        return ct
    ct = sitk.GetArrayFromImage(ct_itk)
    
    ct = torch.tensor(ct).float().unsqueeze(0).unsqueeze(0)
    print(ct.shape)
    if config["modality"] == 'ct':
        ct = scale_intensity_range(ct, a_min=-1000, a_max=1000, b_min=0.0, b_max=1.0, clip=True)
    elif config["modality"] == 'pet':
        ct = scale_intensity_range(ct, a_min=0, a_max=20, b_min=0.0, b_max=1.0, clip=True)
    elif config["modality"] == "mr":
        ct = scale_intensity_range(ct, a_min=0, a_max=3000, b_min=0.0, b_max=1.0, clip=True)

    if isinstance(config["ckpt"],list):
        # print("multi model mode!!")
        wb_preds = []
        brain_preds = []
        for modelnum,oneckpt in enumerate(config["ckpt"]):
            print("model {} inference...".format(modelnum))
            tissueclip = torch.load(os.path.join(current_dir,"tissueclip_RN101.pth")).to("cuda:0")
            model = MPUM(config,tissueclip,1)
            ckpt = torch.load(oneckpt,map_location="cpu")
            new_state_dict = OrderedDict()
            for k, v in ckpt['model'].items():  # 假设权重存储在'ckpt['model']'中
                name = k[7:] if k.startswith('module.') else k  # 移除 'module.' 前缀
                new_state_dict[name] = v
            model.load_state_dict(new_state_dict)
            model = model.to("cuda:0")
            model = model.eval()
            # print(ct.shape)
            # print("----------------sliding_window_inference------------------------")
            with torch.no_grad():
                wb_pred = monai.inferers.sliding_window_inference(
                            ct,(128,128,128),
                            sw_batch_size=1,
                            predictor=model,
                            overlap=0.5,
                            mode="gaussian",
                            sw_device="cuda:0",
                            device="cpu",
                            progress=True)
                wb_pred = torch.sigmoid(wb_pred)
                brain_pred = wb_pred.detach().clone()
                brain_pred[:,1:132] = 0
                if config["tissue"] == "all":
                    wb_pred[:,1] = 0
                    wb_pred[:,4] = 0
                    wb_pred[:,132:] = 0
                elif config["tissue"] == "brain":
                    brain_pred[:,1:132] = 0
                wb_preds.append(wb_pred)
                brain_preds.append(brain_pred)
        wb_pred = 0
        brain_pred = 0
        for www in wb_preds:
            wb_pred += www
        for www in brain_preds:
            brain_pred += www

        if dataset_mapping_to_model is not None:
            dataset_to_model_config = dataset_config_to_rulematrix(dataset_mapping_to_model)
            b,c,z,w,y = wb_pred.shape
            pred = torch.matmul(dataset_to_model_config.float(), wb_pred.view(b,c,-1)).view(b,-1,z,w,y)
            for ii,value in enumerate(torch.sum(dataset_to_model_config,dim=1)):
                if value == 1:
                    continue
                pred[0,ii],_ = torch.max(wb_pred[0][dataset_to_model_config[ii]==1],dim=0)
            wb_pred = pred.float()
            with open(dataset_mapping_to_model,"r") as f:
                d = json.load(f)
            readme = {}
            for key in d:
                readme[len(readme)] = key
        else:
            readme = label215

        combined = torch.argmax(wb_pred[0],dim=0).detach().cpu()
        combined_brain = torch.argmax(brain_pred[0],dim=0).detach().cpu()
        mask = (combined_brain != 0)
        combined[mask] = combined_brain[mask]
        sitk_image = sitk.GetImageFromArray(combined)
        # 设置方向和像素间距
        sitk_image.SetDirection(ct_itk.GetDirection())
        sitk_image.SetSpacing(ct_itk.GetSpacing())
        sitk_image.SetOrigin(ct_itk.GetOrigin())
    
        resampler = sitk.ResampleImageFilter()
        resampler.SetReferenceImage(orict_itk)
        resampler.SetInterpolator(sitk.sitkNearestNeighbor)
        sitk_image = resampler.Execute(sitk_image)
        array = sitk.GetArrayFromImage(sitk_image)
        print(np.max(array))
        print(array.shape)
        if not os.path.exists(output_seg_path):
            os.makedirs(output_seg_path)
            print(f"目录已创建：{output_seg_path}")
        sitk.WriteImage(sitk_image, os.path.join(output_seg_path,"merge.nii.gz"))
        print("create nii.gz ",os.path.join(output_seg_path,"merge.nii.gz"))
    else:
        print("single model mode!!")
        tissueclip = torch.load(os.path.join(current_dir,"tissueclip_RN101.pth")).to("cuda:0")
        print(config)
        model = MPUM(config,tissueclip,1)
    
        print(config["ckpt"])
        ckpt = torch.load(config["ckpt"],map_location="cpu")
        
        new_state_dict = OrderedDict()
        for k, v in ckpt['model'].items():  # 假设权重存储在'ckpt['model']'中
            name = k[7:] if k.startswith('module.') else k  # 移除 'module.' 前缀
            new_state_dict[name] = v
        model.load_state_dict(new_state_dict)
        
        model = model.to("cuda:0")
        model = model.eval()
        print(ct.shape)
        print("----------------sliding_window_inference------------------------")
        with torch.no_grad():
            wb_pred = monai.inferers.sliding_window_inference(
                        ct,(128,128,128),
                        sw_batch_size=1,
                        predictor=model,
                        overlap=0.5,
                        mode="gaussian",
                        sw_device="cuda:0",
                        device="cpu",
                        progress=True)
            # wb_pred = torch.sigmoid(wb_pred)
        print("----------------post-process------------------------")
        
        if dataset_mapping_to_model is not None:
            dataset_to_model_config = dataset_config_to_rulematrix(dataset_mapping_to_model)
            b,c,z,w,y = wb_pred.shape
            pred = torch.matmul(dataset_to_model_config.float(), wb_pred.view(b,c,-1)).view(b,-1,z,w,y)
            for ii,value in enumerate(torch.sum(dataset_to_model_config,dim=1)):
                if value == 1:
                    continue
                pred[0,ii],_ = torch.max(wb_pred[0][dataset_to_model_config[ii]==1],dim=0)
            wb_pred = pred.float()
            with open(dataset_mapping_to_model,"r") as f:
                d = json.load(f)
            readme = {}
            for key in d:
                readme[len(readme)] = key
        else:
            readme = label215
    
        combined = torch.argmax(wb_pred[0],dim=0).detach().cpu()
        sitk_image = sitk.GetImageFromArray(combined)
        # 设置方向和像素间距
        sitk_image.SetDirection(ct_itk.GetDirection())
        sitk_image.SetSpacing(ct_itk.GetSpacing())
        sitk_image.SetOrigin(ct_itk.GetOrigin())
    
        resampler = sitk.ResampleImageFilter()
        resampler.SetReferenceImage(orict_itk)
        resampler.SetInterpolator(sitk.sitkNearestNeighbor)
        sitk_image = resampler.Execute(sitk_image)
        array = sitk.GetArrayFromImage(sitk_image)
        print(np.max(array))
        print(array.shape)
        if not os.path.exists(output_seg_path):
            os.makedirs(output_seg_path)
            print(f"目录已创建：{output_seg_path}")
        sitk.WriteImage(sitk_image, os.path.join(output_seg_path,"merge.nii.gz"))
        print("create nii.gz ",os.path.join(output_seg_path,"merge.nii.gz"))
    with open(os.path.join(output_seg_path,"readme.json"),"w") as f:
        json.dump(readme,f)